@@ -55,6 +55,13 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         self.m_lessonfile = None
         self.m_question = None
         self.m_custom_mode = False
+        self.m_lessonfile_defs = {'yes': True, 'no': False,
+            'hidden': 'hidden',
+            'enabled': 'enabled',
+            'disabled': 'disabled'}
+        # Some exercises modules want other default values for the
+        # lessonfile header variables. They can add them to this dict
+        self.m_lessonfile_header_defaults = {}
     def maybe_auto_new_question(self):
         if self.get_bool('new_question_automatically'):
             if self.m_timeout_handle is None:
@@ -88,17 +95,9 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         if not self.m_lessonfile:
             self.m_P = None
             return
-        self.m_P = self.lessonfileclass()
-        try:
-            self.m_P.parse_file(self.m_lessonfile)
-        # We don't have to check for LessonfileExceptions here because
-        # the global exception hook will catch them.
-        except IOError, e:
-            self.m_P = None
-            solfege.win.display_error_message(
-"""There was an IOError while trying to parse a lesson file:
-%s.""" % e)
-            return
+        self.m_P = self.lessonfileclass(self.m_lessonfile_defs,
+                                        self.m_lessonfile_header_defaults)
+        self.m_P.parse_file(self.m_lessonfile)
         if [q for q in self.m_P.m_questions if isinstance(q.get('music'), lessonfile.Cmdline)]:
             run = gu.dialog_yesno(_("The lessonfile contain potentially dangerous code because it run external programs. Run anyway?"))
             if not run:
@@ -126,7 +125,7 @@ class MelodicIntervalTeacher(Teacher):
     OK = 0
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
-    no_intervals_str = _("No intervals selected for interval number %i.") 
+    no_intervals_str = _("No intervals selected for interval number %i.")
     def __init__(self, exname):
         Teacher.__init__(self, exname)
         self.m_tonika = None
@@ -172,7 +171,7 @@ class MelodicIntervalTeacher(Teacher):
             # will make sure the int value is within the correct limits.
             lock_tonic = mpd.MusicalPitch.new_from_int(self.get_int("lock-to-key-note"))
             lock_scaletype = utils.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
-        
+
         def try_make_question():
             try:
                 if self.get_bool("lock-to-key"):
@@ -351,8 +350,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         self.practise_box = gtk.VBox()
         self.practise_box.show()
         vbox.pack_start(self.practise_box, False)
+
+        box = gtk.VBox()
+        self.practise_box.pack_start(box, False, padding=gu.hig.SPACE_LARGE)
+        box.show()
         self.g_lesson_heading = gtk.Label()
-        self.practise_box.pack_start(self.g_lesson_heading, padding=18)
+        box.pack_start(self.g_lesson_heading)
+
+        self.g_lesson_description = gtk.Label()
+        self.g_lesson_description.set_line_wrap(True)
+        box.pack_start(self.g_lesson_description)
 
         self.action_area = gtk.HBox()
         self.action_area.show()
@@ -394,6 +401,9 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         hbox.pack_start(self.g_deprecated_label)
         self.practise_box.pack_start(hbox, False, False)
         self.practise_box.reorder_child(hbox, 0)
+    def set_deprecation_text(self, oldmodule, newmodule, filename):
+        self.g_deprecated_label.set_line_wrap(True)
+        self.g_deprecated_label.set_markup("<b>The %(oldmodule)s module is deprecated. Convert \"%(filename)s\" to use the %(newmodule)s module.</b>" % locals())
     def std_buttons_add(self, *buttons):
         """
         buttons is a sequence of tuples ('buttonname', callback)
@@ -435,6 +445,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         if [n for n in self._std_buttons if n.split("-")[0] == 'new']:
             self.g_new.set_sensitive(True)
         for b in ('repeat', 'repeat_slowly', 'repeat_arpeggio',
+            'repeat_first', 'repeat_last',
             'repeat_melodic', 'guess_answer', 'play_tonic', 'play_music',
             'play_answer',
             'display_music', 'show', 'give_up'):
@@ -477,6 +488,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_new.set_sensitive(
                 not self.get_bool('config/picky_on_new_question'))
         for s in ('repeat', 'repeat_slowly', 'repeat_arpeggio',
+                'repeat_first', 'repeat_last',
                 'repeat_melodic', 'guess_answer', 'play_music',
                 'display_music', 'play_answer'):
             if s in self._std_buttons:
@@ -484,17 +496,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         if 'play_tonic' in self._std_buttons:
             self.g_play_tonic.set_sensitive(
                 'tonic' in self.m_t.m_P.get_question())
-        if 'show' in self._std_buttons:
-            self.g_show.set_sensitive(False)
-        if 'give_up' in self._std_buttons:
-            self.g_give_up.set_sensitive(False)
+        for n in 'show', 'give_up', 'guess_answer':
+            if n in self._std_buttons:
+                getattr(self, 'g_%s' % n).set_sensitive(False)
     def std_buttons_end_practise(self):
         # We check for the lesson parser because it is possible that a
         # completely broken lesson file was started, and the user tries
         # to exit that exercise or quit the program. And if the lesson file
         # does not have a header block, then we cannot call
         # std_buttons_start_practise
-        if self.m_t.m_P and getattr(self.m_t.m_P, 'header', None):
+        if self.m_t.m_P and hasattr(self.m_t.m_P, 'header'):
             self.std_buttons_start_practise()
         else:
             for name in self._std_buttons:
@@ -553,6 +564,13 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             self.g_lesson_heading.show()
         else:
             self.g_lesson_heading.hide()
+    def set_lesson_description(self, txt):
+        if txt:
+            self.g_lesson_description.set_text(gu.escape(txt))
+            self.g_lesson_description.set_use_markup(True)
+            self.g_lesson_description.show()
+        else:
+            self.g_lesson_description.hide()
     def on_start_practise(self):
         """
         Code that are common for all exercises. Not used by many now,
@@ -562,8 +580,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
         self.handle_statistics_page_sensibility()
         if self.get_bool("lock-to-key"):
             self.set_bool("lock-to-key", False)
-        if getattr(self, 'g_flashbar', None):
+        if hasattr(self, 'g_flashbar'):
             self.g_flashbar.require_size(["|Xy"])
+        if self.m_t.m_P.header.lesson_heading:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        else:
+            self.set_lesson_heading(getattr(self, "lesson_heading", None))
+        if self.m_t.m_P.header.lesson_description:
+            self.set_lesson_description(self.m_t.m_P.header.lesson_description)
+        else:
+            self.set_lesson_description(getattr(self, "lesson_description", None))
     def on_end_practise(self):
         pass
     def handle_config_box_visibility(self):
@@ -862,14 +888,9 @@ class IntervalGui(Gui):
             v = []
             for x in range(self.get_int('maximum_number_of_intervals')):
                 v.append('ask_for_intervals_%i' % x)
-        if i == 0:
-            assert inputwidgets.inputwidget_names[i] == _("Buttons")
-            self.g_input = inputwidgets.IntervalButtonsWidget(self.m_exname,
-                  'intervals', self.click_on_interval, self.get_interval_input_list, v)
-        else:
-            self.g_input = inputwidgets.name_to_inputwidget(
-                                 inputwidgets.inputwidget_names[i],
-                                 self.click_on_interval)
+        self.g_input = inputwidgets.int_to_inputwidget(
+            i, self.click_on_interval, self.m_exname,
+            self.get_interval_input_list, v)
         self.practise_box.pack_start(self.g_input)
         self.practise_box.reorder_child(self.g_input, 1)
         self.g_input.show()
@@ -877,7 +898,8 @@ class IntervalGui(Gui):
             # Don't call on_end_practise if we are starting up the exercise.
             # This whole thing is a mess.
             self.on_end_practise()
-        self.g_disable_unused_buttons.set_sensitive(self.get_int('inputwidget')==0)
+        self.g_disable_unused_buttons.set_sensitive(
+            isinstance(self.g_input, inputwidgets.IntervalButtonsWidgetBase))
     def setup_key_bindings(self):
         keys = ['minor2', 'major2', 'minor3', 'major3',
                 'perfect4', 'diminished5', 'perfect5', 'minor6',
